/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    robdekon
 * @author     Christoph Pohl ( christoph dot pohl at kit dot edu )
 * @date       06.09.22
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <experimental/memory>

#include <ArmarXCore/core/application/properties/forward_declarations.h>

#include <RobotAPI/libraries/aron/core/data/visitor/variant/VariantVisitor.h>

namespace armarx::aron::component_config
{

    class PropertyDefinitionSetterVisitor : public aron::data::RecursiveConstTypedVariantVisitor
    {
    public:
        explicit PropertyDefinitionSetterVisitor(const armarx::PropertyDefinitionsPtr& defs);

        void visitAronVariant(const data::IntPtr&, const type::IntEnumPtr&) override;

        void visitAronVariant(const data::IntPtr&, const type::IntPtr&) override;

        void visitAronVariant(const data::FloatPtr&, const type::FloatPtr&) override;

        void visitAronVariant(const data::DoublePtr&, const type::DoublePtr&) override;

        void visitAronVariant(const data::BoolPtr&, const type::BoolPtr&) override;

        void visitAronVariant(const data::StringPtr&, const type::StringPtr&) override;

        void visitAronVariantOnEnter(const data::ListPtr&, const type::ListPtr&) override;

        void visitAronVariantOnExit(const data::ListPtr&, const type::ListPtr&) override;

        void visitAronVariantOnEnter(const data::DictPtr&, const type::DictPtr&) override;

        void visitAronVariantOnExit(const data::DictPtr&, const type::DictPtr&) override;

        void visitAronVariantOnEnter(const data::DictPtr& ptr,
                                     const type::ObjectPtr& objectPtr) override;

    private:
        std::experimental::observer_ptr<PropertyDefinitionContainer> property_definitions_;
        std::string global_namespace_{""};
        std::atomic<bool> in_list_{false};
    };

    class PropertyDefinitionGetterVisitor :
        public aron::data::RecursiveTypedVisitor<data::VariantPtr, const type::VariantPtr>
    {
    public:
        explicit PropertyDefinitionGetterVisitor(const armarx::PropertyUser& defs);

        type::Descriptor getDescriptor(DataInput& o, TypeInput& t) override;

        MapElements getObjectElements(DataInput& o, TypeInput& t) override;

        MapElements getDictElements(DataInput& o, TypeInput& t) override;

        ListElements getListElements(DataInput& o, TypeInput& t) override;

        PairElements getPairElements(DataInput& o, TypeInput& t) override;

        TupleElements getTupleElements(DataInput& o, TypeInput& t) override;

        void visitInt(DataInput& elementData, TypeInput& elementType) override;

        void visitFloat(DataInput& elementData, TypeInput& elementType) override;

        void visitDouble(DataInput& elementData, TypeInput& elementType) override;

        void visitBool(DataInput& elementData, TypeInput& elementType) override;

        void visitString(DataInput& elementData, TypeInput& elementType) override;

        void visitUnknown(DataInput& elementData, TypeInput& elementType) override;

        void visitListOnEnter(DataInput& elementData, TypeInput& elementType) override;

        void visitListOnExit(DataInput& elementData, TypeInput& elementType) override;

        void visitDictOnEnter(DataInput& elementData, TypeInput& elementType) override;

        void visitDictOnExit(DataInput& elementData, TypeInput& elementType) override;

        void visitObjectOnEnter(DataInput& elementData, TypeInput& elementType) override;

        void visitIntEnum(DataInput& elementData, TypeInput& elementType) override;

    private:
        std::experimental::observer_ptr<const armarx::PropertyUser> property_user_;
        std::string global_namespace_{""};
        std::atomic<bool> in_list_;
    };

} // namespace armarx::aron::component_config